% GUI for input needed to create inputs for validatetsv.
%
% Usage:
%
%   >>  validatetsv_input(tab)
%
% Output:
%
%   tab
%                    The 'Validate' tab object in pop_tsv.
%                    
%
% Copyright (C) 2012-2016 Thomas Rognon tcrognon@gmail.com,
% Jeremy Cockfield jeremy.cockfield@gmail.com, and
% Kay Robbins kay.robbins@utsa.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

function validatetsv_input(tab)
latestHED = 'HED.xml';
hedXMLPath = which(latestHED);
hedXML = hedXMLPath;
replaceFile = '';
tsvFile = '';
outDir = pwd;
tsvTagColumns = 2;
hasHeader = true;
generateWarnings = false;
hedXMLCtrl = '';
tsvFileCtrl = '';
remapFileCtrl = '';
outputDirectoryCtrl = '';
createPanel(tab);

    function browseHedXMLCallback(src, eventdata, hedCtrl, ...
            myTitle) %#ok<INUSL>
        % Callback for 'Browse' button that sets the 'HED' editbox
        [tFile, tPath] = uigetfile({'*.xml', 'XML files (*.xml)'}, ...
            myTitle);
        if tFile ~= 0
            hedXML = fullfile(tPath, tFile);
            set(hedCtrl, 'String', hedXML);
        end
    end % browseHedXMLCallback

    function browseOutputDirectoryCallback(~, ~, outputTxtCtrl, myTitle)
        % Callback for 'Browse' button that sets the 'Output directory'
        % editbox
        startPath = get(outputTxtCtrl, 'String');
        if isempty(startPath) || ~ischar(startPath) || ~isdir(startPath)
            startPath = pwd;
        end
        dName = uigetdir(startPath, myTitle);
        if dName ~=0
            setOutputDirectory(dName);
        end
    end % browseOutputDirectoryCallback

    function browseTSVFileCallback(~, ~, tagsCtrl, myTitle)
        % Callback for 'Browse' button that sets the 'Tags' editbox
        [tFile, tPath] = uigetfile({'*.tsv', 'Tab-separated files'; ...
            '*.txt', 'Text files'; '*.*', 'All files'}, myTitle);
        if tFile ~= 0
            tsvFile = fullfile(tPath, tFile);
            set(tagsCtrl, 'String', tsvFile);
        end
    end % browseTSVFileCallback

    function browseReplaceFileCallback(~, ~, remapCtrl, myTitle) 
        % Callback for 'Browse' button that sets the 'Tags' editbox
        [tFile, tPath] = uigetfile({'*.tsv', 'Tab-separated files'; ...
            '*.txt', 'Text files'; '*.*', 'All files'}, myTitle);
        if tFile ~= 0
            replaceFile = fullfile(tPath, tFile);
            set(remapCtrl, 'String', replaceFile);
        end
    end % browseReplaceFileCallback

    function createButtons(panel)
        % Creates the buttons in the panel
        uicontrol('Parent', panel, ...
            'String', 'Browse', ...
            'Style', 'pushbutton', ...
            'TooltipString', 'Press to choose a HED schema', ...
            'Units','normalized',...
            'Callback', {@browseHedXMLCallback, ...
            hedXMLCtrl, 'Browse for HED XML file'}, ...
            'Position', [0.775 0.9 0.2 0.1]);
        uicontrol('Parent', panel, ...
            'String', 'Browse', ...
            'Style', 'pushbutton', ...
            'TooltipString', 'Press to choose a tab-separated file', ...
            'Units','normalized',...
            'Callback', {@browseTSVFileCallback, ...
            tsvFileCtrl, 'Browse for tab-separated file'}, ...
            'Position', [0.775 0.75 0.2 0.1]);
        uicontrol('Parent', panel, ...
            'String', 'Browse', ...
            'Style', 'pushbutton', ...
            'TooltipString', ...
            'Press to choose a replacement file', ...
            'Units','normalized',...
            'Callback', {@browseReplaceFileCallback, ...
            remapFileCtrl, ...
            'Browse for map file'}, ...
            'Position', [0.775 0.6 0.2 0.1]);
        uicontrol('Parent', panel, ...
            'String', 'Browse', ...
            'Style', 'pushbutton', ...
            'TooltipString', ...
            'Press to choose a output directory', ...
            'Units','normalized',...
            'Callback', {@browseOutputDirectoryCallback, ...
            outputDirectoryCtrl, ...
            'Browse for ouput directory'}, ...
            'Position', [0.775 0.45 0.2 0.1]);
        uicontrol('Parent', panel, ...
            'String', 'Help', ...
            'Style', 'pushbutton', ...
            'TooltipString', ...
            'Press for instructions.', ...
            'Units','normalized',...
            'Callback', {@helpCallback}, ...
            'Position', [0.775 0.3 0.2 0.1]);
        uicontrol('Parent', panel, ...
            'String', 'Validate', ...
            'Style', 'pushbutton', ...
            'TooltipString', ...
            'Press when finished', ...
            'Units','normalized',...
            'Callback', {@validateTSVCallback}, ...
            'Position', [0.775 0.025 0.2 0.1]);
    end % createButtons

    function createCheckboxes()
        % Creates the checkboxes in the panel
        panel = uipanel('Parent', tab, ...
            'BackgroundColor', [.94 .94 .94], ...
            'FontSize', 12, ...
            'Position', [0.15 0.02 0.6 0.2], ...
            'Title', 'Additional options');
        uicontrol('Parent', panel, ...
            'Style', 'checkbox', ...
            'HorizontalAlignment', 'Left', ...
            'String', 'Tab-separated input file has a header', ...
            'Value', hasHeader, ...
            'TooltipString', ['Check if the tab-separated input file' ...
            ' has a header, uncheck if it does not have a header.'], ...
            'Units','normalized',...
            'Callback', {@hasHeaderCallback}, ...
            'Position', [0.04 0.6 .9 0.4]);
        uicontrol('Parent', panel, ...
            'Style', 'checkbox', ...
            'HorizontalAlignment', 'Left', ...
            'String', 'Include warnings in log file', ...
            'Value', generateWarnings, ...
            'TooltipString', ['Check to include warnings in the log' ...
            ' file in addition to errors. If unchecked only errors are' ...
            ' included in the log file'], ...
            'Units','normalized',...
            'Callback', {@genearteWarningsCallback}, ...
            'Position', [0.04 0.1 0.9 0.4]);
    end % createCheckboxes

    function createEditBoxes(panel)
        % Creates the edit boxes in the panel
        hedXMLCtrl = uicontrol('Parent', panel, ...
            'Style', 'edit', ...
            'BackgroundColor', 'w', ...
            'HorizontalAlignment', 'Left', ...
            'String', hedXMLPath, ...
            'TooltipString', 'A HED XML file used for validation.', ...
            'Units','normalized',...
            'Callback', {@hedEditboxCallback}, ...
            'Position', [0.15 0.9 0.6 0.1]);
        tsvFileCtrl = uicontrol('Parent', panel, ...
            'Style', 'edit', ...
            'BackgroundColor', 'w', ...
            'HorizontalAlignment', 'Left', ...
            'String', '', ...
            'TooltipString', ['A tab-separated input file containing' ...
            ' HED tags.'], ...
            'Units','normalized',...
            'Callback', {@tsvEditboxCallback}, ...
            'Position', [0.15 0.75 0.6 0.1]);
        remapFileCtrl = uicontrol('Parent', panel, ...
            'Style', 'edit', ...
            'BackgroundColor', 'w', ...
            'HorizontalAlignment', 'Left', ...
            'String', '', ...
            'TooltipString', ['A tab-separated replacement file containing' ...
            ' tags that generated issues from a previous validation.' ...
            ' Any tags not in this file that generate issues will be' ...
            ' appended to first column.'], ...
            'Units','normalized',...
            'Callback', {@replaceEditboxCallback}, ...
            'Position', [0.15 0.6 0.6 0.1]);
        outputDirectoryCtrl = uicontrol('Parent', panel, ...
            'Style', 'edit', ...
            'BackgroundColor', 'w', ...
            'HorizontalAlignment', 'Left', ...
            'String', pwd, ...
            'TooltipString', ['A directory where the validation output' ...
            ' is written to if the ''Write Output'' checkbox' ...
            ' is checked.'], ...
            'Units','normalized',...
            'Callback', {@outputEditboxCallback}, ...
            'Position', [0.15 0.45 0.6 0.1]);
        uicontrol('Parent', panel, ...
            'Style', 'edit', ...
            'BackgroundColor', 'w', ...
            'HorizontalAlignment', 'Left', ...
            'Tag', 'validateHedXmlEdit', ...
            'String', '2', ...
            'TooltipString', ['The tag columns in a tab-separated' ...
            ' input file. The columns can be specified as a single' ...
            ' number or a comma separated list of numbers (e.g. 2 or' ...
            ' 2,3,4)'], ...
            'Units','normalized',...
            'Callback', {@tsvTagColumnsCtrlCallback}, ...
            'Position', [0.15 0.3 0.6 0.1]);
    end % createEditBoxes

    function createLabels(panel)
        % Creates the labels in the panel
        uicontrol('parent', panel, ...
            'Style', 'Text', ...
            'Units', 'normalized', ...
            'String', 'HED file', ...
            'HorizontalAlignment', 'Left', ...
            'Position', [0 0.9 0.12 0.08]);
        uicontrol('parent', panel, ...
            'Style', 'Text', ...
            'Units', 'normalized', ...
            'String', 'TSV input file', ...
            'HorizontalAlignment', 'Left', ...
            'Position', [0 0.75 0.12 0.08]);
        uicontrol('parent', panel, ...
            'Style', 'Text', ...
            'Units', 'normalized', ...
            'String', 'Replace file (optional)', ...
            'HorizontalAlignment', 'Left', ...
            'Position', [0 0.6 0.15 0.08]);
        uicontrol('parent', panel, ...
            'Style', 'Text', ...
            'Units', 'normalized', ...
            'String', 'Output directory', ...
            'HorizontalAlignment', 'Left', ...
            'Position', [0 0.45 0.12 0.1]);
        uicontrol('parent', panel, ...
            'Style', 'Text', ...
            'Units', 'normalized', ...
            'String', 'HED tag columns', ...
            'HorizontalAlignment', 'Left', ...
            'Position', [0 0.3 0.12 0.08]);
    end % createLabels

    function createPanel(tab)
        % Creates the 'Validate Tags' tab panel
        panel = uipanel('Parent', tab, ...
            'BorderType', 'none', ...
            'BackgroundColor', [.94 .94 .94], ...
            'FontSize', 12, ...
            'Position', [0 0 1 1]);
        createLabels(panel);
        createEditBoxes(panel);
        createCheckboxes();
        createButtons(panel);
    end % createPanel

    function tsvTagColumnsCtrlCallback(src, ~)
        % Callback for user directly editing the 'HED tag columns' editbox
        tsvTagColumns = str2num(get(src, 'String')); %#ok<ST2NM>
    end % tsvTagColumnsCtrlCallback

    function hasHeaderCallback(src, ~) 
        % Callback for user directly editing the 'Tab-separated input file
        % has a header' checkbox
        hasHeader = get(src, 'Max') == get(src, 'Value');
    end % hasHeaderCallback

    function genearteWarningsCallback(src, ~) 
        % Callback for user directly editing the 'Include warnings in log
        % file' checkbox
        generateWarnings = get(src, 'Max') == get(src, 'Value');
    end % genearteWarningsCallback

    function hedEditboxCallback(src, ~) 
        % Callback for user directly editing the 'HED file' editbox
        hedXML = get(src, 'String');
    end % hedEditboxCallback

    function replaceEditboxCallback(src, ~) 
        % Callback for user directly editing the 'Replace file' editbox
        replaceFile = get(src, 'String');
    end % replaceEditboxCallback

    function outputEditboxCallback(src, ~) 
        % Callback for user directly editing the 'Output directory' editbox
        outDir = get(src, 'String');
    end % outputEditboxCallback

    function setOutputDirectory(dName)
        % Sets the 'Output' edit box based on the 'Tags' editbox
        outDir = dName;
        set(outputDirectoryCtrl, 'String', outDir);
    end % setOutputDirectory

    function tsvEditboxCallback(src, ~) 
        % Callback for user directly editing the 'Tags' editbox
        tsvFile = get(src, 'String');
    end % tsvEditboxCallback

    function helpCallback(~, ~) 
        % Callback for the 'Validate' button
        helpdlg(sprintf(['Validates the HED tags in a tab-separated' ...
            ' file against a HED schema. \n\n***Main Options***' ...
            ' \n\nHED file - A XML file containing every single' ...
            ' HED tag and its attributes. This by default will be' ...
            ' the HED.xml file found in the hed directory. If you want' ...
            ' to be sure that you are using the latest version click' ...
            ' the ''Check For Latest HED'' tab and press the' ...
            ' ''Update'' button to check.\n\nTSV input file - A' ...
            ' tab-separated file containing HED tags in a single' ...
            ' column or multiple columns.\n\nReplace file -' ...
            ' A optional two column tab-separated file used to find' ...
            ' and replace the HED tags in the input file. This file' ...
            ' will be for later use when using the' ...
            ' ''Find and Replace'' tab. The first column will be the' ...
            ' HED tags to find and the second column will be HED tags' ...
            ' that will replace them. If a replace file is provided' ...
            ' then any HED tags not already in the first column of' ...
            ' this file that generate issues from the validation will' ...
            ' be appended to the end of it. Reusing a replace file' ...
            ' comes in handy when you have multiple tab-separated' ...
            ' files that have the same HED tags or you simply want to' ...
            ' consolidate all of the changes in one file instead of' ...
            ' several.\n\nOutput directory - The directory where' ...
            ' the output files will be written to. There will be log' ...
            ' file containing any issues that were found while' ...
            ' validating the HED tags. If there were issues found then' ...
            ' a replace file will be created in addition to the' ...
            ' log file if an optional one isn''t already provided.' ...
            ' The default output directory will be the current' ...
            ' directory.\n\nHED tag columns - The columns in the' ...
            ' tab-separated input file that contains the HED tags.' ...
            ' This is to be specified with a single number or a comma' ...
            ' separated list of numbers (e.g. 1 or 1,2,3,4). The' ...
            ' default will be the second column. ' ...
            ' \n\n***Additional Options*** \n\nTab-separated input' ...
            ' file has header - Check if the the tab-separated input' ...
            ' file has a header. The first row will not be validated,' ...
            ' otherwise it will and this can generate issues.''' ...
            ' \n\nInclude warnings in log file - Check to include' ...
            ' warnings in the log file in addition to errors. If' ...
            ' unchecked only errors are included in the log file.']), ...
            'Validation Instructions');
    end % helpCallback

    function validateTSVCallback(~, ~) 
        % Callback for the 'Validate' button
        if isempty(hedXML)
            errordlg('HED XML file is empty', 'Input required', 'modal');
        elseif isempty(tsvFile)
            errordlg('Tab-separated input file is empty', ...
                'Input required', 'modal');
        elseif isempty(outDir)
            errordlg('Output directory is empty', 'Input required', ...
                'modal');
        elseif isempty(tsvTagColumns)
            errordlg('HED tag columns are empty', 'Input required', ...
                'modal');
        else
            wb = waitbar(.5,'Please wait...');
            try
                validatetsv(tsvFile, tsvTagColumns, ...
                    'hedXML', hedXML, 'outDir', outDir, ...
                    'hasHeader', hasHeader, 'writeOutput', true, ...
                    'generateWarnings', generateWarnings);
                msgbox('Complete!');
                close(wb);
            catch ME
                close(wb);
                errordlg(ME.message);
            end
        end
    end % validateTSVCallback

end % validatetsv_input